@@ -4,6 +4,14 @@
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE UnboxedTuples #-}
 {-# LANGUAGE PatternGuards #-}
+{-# LANGUAGE Trustworthy #-}
+
+#if __GLASGOW_HASKELL__ >= 800
+{-# LANGUAGE DeriveLift #-}
+{-# LANGUAGE StandaloneDeriving #-}
+#else
+{-# LANGUAGE TemplateHaskell #-}
+#endif
 
 -- |
 -- Module      :  Data.Scientific
@@ -48,7 +56,7 @@
 --
 -- This module is designed to be imported qualified:
 --
--- @import Data.Scientific as Scientific@
+-- @import qualified Data.Scientific as Scientific@
 module Data.Scientific
     ( Scientific
 
@@ -98,7 +106,6 @@ module Data.Scientific
 
 import           Control.Exception            (throw, ArithException(DivideByZero))
 import           Control.Monad                (mplus)
-import           Control.Monad.ST             (runST)
 import           Control.DeepSeq              (NFData, rnf)
 import           Data.Binary                  (Binary, get, put)
 import           Data.Char                    (intToDigit, ord)
@@ -108,7 +115,6 @@ import           Data.Int                     (Int8, Int16, Int32, Int64)
 import qualified Data.Map            as M     (Map, empty, insert, lookup)
 import           Data.Ratio                   ((%), numerator, denominator)
 import           Data.Typeable                (Typeable)
-import qualified Data.Primitive.Array as Primitive
 import           Data.Word                    (Word8, Word16, Word32, Word64)
 import           Math.NumberTheory.Logarithms (integerLog10')
 import qualified Numeric                      (floatToDigits)
@@ -129,16 +135,10 @@ import           Data.Word                    (Word)
 import           Control.Applicative          ((<*>))
 #endif
 
-#if MIN_VERSION_base(4,5,0)
-import           Data.Bits                    (unsafeShiftR)
-#else
-import           Data.Bits                    (shiftR)
-#endif
-
-import GHC.Integer        (quotRemInteger, quotInteger)
-import GHC.Integer.Compat (divInteger)
-import Utils              (roundTo)
+import GHC.Integer.Compat (quotRemInteger, quotInteger, divInteger)
+import Utils              (maxExpt, roundTo, magnitude)
 
+import Language.Haskell.TH.Syntax (Lift (..))
 
 ----------------------------------------------------------------------
 -- Type
@@ -163,6 +163,20 @@ data Scientific = Scientific
       -- in 'toDecimalDigits'.
       --
       -- Use 'normalize' to do manual normalization.
+      --
+      -- /WARNING:/ 'coefficient' and 'base10exponent' violate
+      -- substantivity of 'Eq'.
+      --
+      -- >>> let x = scientific 1 2
+      -- >>> let y = scientific 100 0
+      -- >>> x == y
+      -- True
+      --
+      -- but
+      --
+      -- >>> (coefficient x == coefficient y, base10Exponent x == base10Exponent y)
+      -- (False,False)
+      --
 
     , base10Exponent :: {-# UNPACK #-} !Int
       -- ^ The base-10 exponent of a scientific number.
@@ -173,17 +187,32 @@ data Scientific = Scientific
 scientific :: Integer -> Int -> Scientific
 scientific = Scientific
 
-
 ----------------------------------------------------------------------
 -- Instances
 ----------------------------------------------------------------------
 
+#if __GLASGOW_HASKELL__ >= 800
+-- | @since 0.3.7.0
+deriving instance Lift Scientific
+#else
+instance Lift Scientific where
+    lift (Scientific c e) = [| Scientific c e |]
+#endif
+
+
 instance NFData Scientific where
     rnf (Scientific _ _) = ()
 
 -- | A hash can be safely calculated from a @Scientific@. No magnitude @10^e@ is
 -- calculated so there's no risk of a blowup in space or time when hashing
 -- scientific numbers coming from untrusted sources.
+--
+-- >>> import Data.Hashable (hash)
+-- >>> let x = scientific 1 2
+-- >>> let y = scientific 100 0
+-- >>> (x == y, hash x == hash y)
+-- (True,True)
+--
 instance Hashable Scientific where
     hashWithSalt salt s = salt `hashWithSalt` c `hashWithSalt` e
       where
@@ -295,15 +324,23 @@ instance Real Scientific where
 -- | /WARNING:/ 'recip' and '/' will throw an error when their outputs are
 -- <https://en.wikipedia.org/wiki/Repeating_decimal repeating decimals>.
 --
+-- These methods also compute 'Integer' magnitudes (@10^e@). If these methods
+-- are applied to arguments which have huge exponents this could fill up all
+-- space and crash your program! So don't apply these methods to scientific
+-- numbers coming from untrusted sources.
+--
 -- 'fromRational' will throw an error when the input 'Rational' is a repeating
 -- decimal.  Consider using 'fromRationalRepetend' for these rationals which
 -- will detect the repetition and indicate where it starts.
 instance Fractional Scientific where
     recip = fromRational . recip . toRational
-    {-# INLINABLE recip #-}
 
-    x / y = fromRational $ toRational x / toRational y
-    {-# INLINABLE (/) #-}
+    Scientific c1 e1 / Scientific c2 e2
+        | d < 0     = fromRational (x / (fromInteger (magnitude (-d))))
+        | otherwise = fromRational (x *  fromInteger (magnitude   d))
+      where
+        d = e1 - e2
+        x = c1 % c2
 
     fromRational rational =
         case mbRepetendIx of
@@ -651,46 +688,8 @@ toIntegral (Scientific c e) = fromInteger c * magnitude e
 {-# INLINE toIntegral #-}
 
 
-----------------------------------------------------------------------
--- Exponentiation with a cache for the most common numbers.
-----------------------------------------------------------------------
 
--- | The same limit as in GHC.Float.
-maxExpt :: Int
-maxExpt = 324
 
-expts10 :: Primitive.Array Integer
-expts10 = runST $ do
-    ma <- Primitive.newArray maxExpt uninitialised
-    Primitive.writeArray ma 0  1
-    Primitive.writeArray ma 1 10
-    let go !ix
-          | ix == maxExpt = Primitive.unsafeFreezeArray ma
-          | otherwise = do
-              Primitive.writeArray ma  ix        xx
-              Primitive.writeArray ma (ix+1) (10*xx)
-              go (ix+2)
-          where
-            xx = x * x
-            x  = Primitive.indexArray expts10 half
-#if MIN_VERSION_base(4,5,0)
-            !half = ix `unsafeShiftR` 1
-#else
-            !half = ix `shiftR` 1
-#endif
-    go 2
-
-uninitialised :: error
-uninitialised = error "Data.Scientific: uninitialised element"
-
--- | @magnitude e == 10 ^ e@
-magnitude :: Num a => Int -> a
-magnitude e | e < maxExpt = cachedPow10 e
-            | otherwise   = cachedPow10 hi * 10 ^ (e - hi)
-    where
-      cachedPow10 = fromInteger . Primitive.indexArray expts10
-
-      hi = maxExpt - 1
 
 
 ----------------------------------------------------------------------
@@ -979,11 +978,16 @@ isE c = c == 'e' || c == 'E'
 
 -- | See 'formatScientific' if you need more control over the rendering.
 instance Show Scientific where
-    show s | coefficient s < 0 = '-':showPositive (-s)
-           | otherwise         =     showPositive   s
+    showsPrec d s
+        | coefficient s < 0 = showParen (d > prefixMinusPrec) $
+               showChar '-' . showPositive (-s)
+        | otherwise         = showPositive   s
       where
-        showPositive :: Scientific -> String
-        showPositive = fmtAsGeneric . toDecimalDigits
+        prefixMinusPrec :: Int
+        prefixMinusPrec = 6
+
+        showPositive :: Scientific -> ShowS
+        showPositive = showString . fmtAsGeneric . toDecimalDigits
 
         fmtAsGeneric :: ([Int], Int) -> String
         fmtAsGeneric x@(_is, e)
