@@ -18,7 +18,6 @@ import           Data.Int
 import           Data.Word
 import           Data.Scientific                    as Scientific
 import           Test.Tasty
-import           Test.Tasty.Runners.AntXML
 import           Test.Tasty.HUnit                          (testCase, (@?=), Assertion, assertBool)
 import qualified Test.SmallCheck                    as SC
 import qualified Test.SmallCheck.Series             as SC
@@ -114,6 +113,9 @@ main = testMain $ testGroup "scientific"
 
   , testGroup "Formatting"
     [ testProperty "read . show == id" $ \s -> read (show s) === s
+    , testCase "show (Just 1)"    $ testShow (Just 1)    "Just 1.0"
+    , testCase "show (Just 0)"    $ testShow (Just 0)    "Just 0.0"
+    , testCase "show (Just (-1))" $ testShow (Just (-1)) "Just (-1.0)"
 
     , testGroup "toDecimalDigits"
       [ smallQuick "laws"
@@ -279,7 +281,7 @@ main = testMain $ testGroup "scientific"
   ]
 
 testMain :: TestTree -> IO ()
-testMain = defaultMainWithIngredients (antXMLRunner:defaultIngredients)
+testMain = defaultMainWithIngredients defaultIngredients
 
 testReads :: String -> [(Scientific, String)] -> Assertion
 testReads inp out = reads inp @?= out
@@ -287,6 +289,9 @@ testReads inp out = reads inp @?= out
 testRead :: String -> Scientific -> Assertion
 testRead inp out = read inp @?= out
 
+testShow :: Maybe Scientific -> String -> Assertion
+testShow inp out = show inp @?= out
+
 testScientificP :: String -> [(Scientific, String)] -> Assertion
 testScientificP inp out = readP_to_S Scientific.scientificP inp @?= out
 
